#include <windows.h>
#include <assert.h>
#include <fstream.h>
#include "WriteRemoteFile.h"


#define APP_NAME    "Umlaute-Konverter: RemoteFile-Test"

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int nCmdShow)
{
    // berprfen, ob Umlaute-Konverter installiert ist, und wenn
    // Ja, den Pfad der Programmdatei auslesen und in szUK_Path ablegen
    char    szSubKey[] = "Software\\Henning\\UmlauteKonverter\\";
    HKEY    hKey;
    char    szValueName[] = "szProgPath";
    DWORD   dwUK_PathLen = 0;
    LONG    lRegResult;
    lRegResult = 
        RegOpenKeyEx(HKEY_CURRENT_USER, szSubKey, 0, KEY_QUERY_VALUE, &hKey);
    RegQueryValueEx(hKey, szValueName, NULL, NULL, NULL, &dwUK_PathLen);
    if ( (lRegResult != ERROR_SUCCESS) || (dwUK_PathLen == 0) )
    {
        MessageBox(NULL, "Umlaute-Konverter wurde nicht gefunden. Sollte Umlaute-Konverter installiert sein, so starten Sie es bitte einmalig, um die Eintrge in der Registry zu erstellen!", APP_NAME, MB_OK);
        return 0;
    }
    char    * szUK_Path = new char[dwUK_PathLen + 2];
    RegQueryValueEx(hKey, szValueName, NULL, NULL,
                    (unsigned char*)szUK_Path, &dwUK_PathLen);
    RegCloseKey(hKey);

    // Quell-Datei erstellen:
    char        szSrcFileName[] = "c:\\UK_Source.txt";
    char        szDestFileName[] = "c:\\UK_Dest.txt";
    ofstream    OutFile;
    OutFile.open(szSrcFileName, ios::out | ios::trunc );
    OutFile << "Beispieltext fuer Umlaute-Konverter:\n\nAsz er den Aszendenten oder sauere Fruechte?\n";
    OutFile.close();

    // Remote-Datei erstellen:
    CRemoteFile RF;
    // Name und Registrationsschlssel setzen, falls vorhanden
    RF.SetNameAndRegKey("", "");
    // Name von Ein- und Ausgabedatei setzen
    RF.SetFileNames(szSrcFileName, szDestFileName);
    // Ein- und Ausgabeformat setzen
    // Der Eingabetyp umlautetyp_undefined bewirkt, dass
    // UmlauteKonverter versucht, das Format des Eingabetextes
    // selbststndig herauszufinden
    RF.SetFileTypes(umlautetyp_undefined, umlautetyp_WINDOWS);
    // Handles der Nachrichtenfenster setzen (Wird hier nicht bentigt)
    RF.SetWindowHandles(NULL, NULL);
    // Remote-File schreiben
    char    szRFName[] = "c:\\UK_Remote.urf";
    RF.WriteRemoteFile(szRFName);

    // Um Probleme zu vermeiden, muss sichergestellt werden,
    // dass die Datei szDestFileName nicht existiert!
    DeleteFile(szDestFileName);

    // Kommandozeile fr UmlauteKonverter erstellen und
    // UmlauteKonverter starten
    char    * szCmdLine = new char[dwUK_PathLen + 1 + sizeof(szRFName) + 1];
    lstrcpy(szCmdLine, szUK_Path);
    lstrcat(szCmdLine, " ");
    lstrcat(szCmdLine, szRFName);
    STARTUPINFO         si;
    GetStartupInfo(&si);
    PROCESS_INFORMATION pi;
    if ( CreateProcess(szUK_Path, szCmdLine, NULL, NULL, FALSE,
                       NORMAL_PRIORITY_CLASS, NULL, NULL, &si, &pi) == 0 )
    {
        MessageBox(NULL, "Aufruf von Umlaute-Konverter fehlgeschlagen!", APP_NAME, MB_OK);
        return 0;
    }

    // Warten, bis UmlauteKonverter gestartet (D.h., Prozess existiert)
    DWORD   dwExitCode;
    while (GetExitCodeProcess(pi.hProcess, &dwExitCode) == 0 ) ;

    // Warten, bis UmlauteKonverter beendet
    while ( dwExitCode == STILL_ACTIVE )
        GetExitCodeProcess(pi.hProcess, &dwExitCode);

    // An dieser Stelle knnte man nun die Datei szDestFileName
    // zur weiteren Verarbeitung einlesen und sie, sowie gegebenenfalls
    // das Remote-File, lschen

    // Speicher freigeben:
    delete [] szCmdLine;
    delete [] szUK_Path;

    // Erfolgsmeldung ausgeben
    MessageBox(NULL, "Umwandlung erfolgreich! Siehe Dateien c:\\UK_*.*", APP_NAME, MB_OK);

    return 0;
}

